#!/usr/bin/env python

import unittest
import rolodexer
import json

sample_input = """
Booker T., Washington, 87360, 373 781 7380, yellow
Chandler, Kerri, (623)-668-9293, pink, 123123121
James Murphy, yellow, 83880, 018 154 6474
asdfawefawea
""".strip()

sample_output = """
{
  "entries": [
    {
      "color": "yellow",
      "firstname": "James",
      "lastname": "Murphy",
      "phonenumber": "018-154-6474",
      "zipcode": "83880"
    },
    {
      "color": "yellow",
      "firstname": "Booker T.",
      "lastname": "Washington",
      "phonenumber": "373-781-7380",
      "zipcode": "87360"
    }
  ],
  "errors": [
    1,
    3
  ]
}
""".strip()

class RolodexerTests(unittest.TestCase):
    
    def setUp(self):
        self.maxDiff = 1000
    
    def test_tokenize(self):
        # all of these should tokenize (even with invalid individual terms)
        line0 = 'Booker T., Washington, 87360, 373 781 7380, yellow'
        line1 = 'Chandler, Kerri, (623)-668-9293, pink, 123123121'
        line2 = 'James Murphy, yellow, 83880, 018 154 6474'
        
        terms0 = rolodexer.tokenize(line0)
        terms1 = rolodexer.tokenize(line1)
        terms2 = rolodexer.tokenize(line2)
        
        self.assertEqual(len(terms0), 5)
        self.assertEqual(len(terms1), 5)
        self.assertEqual(len(terms2), 4) # first/last are single term
    
    def test_identify_color(self):
        self.assertTrue(rolodexer.is_color('yellow'))
        self.assertTrue(rolodexer.is_color('pink'))
        self.assertTrue(rolodexer.is_color('aqua marine'))
        
        self.assertFalse(rolodexer.is_color('Booker T.'))
        self.assertFalse(rolodexer.is_color('Washington'))
        self.assertFalse(rolodexer.is_color('Chandler'))
        self.assertFalse(rolodexer.is_color('Kerri'))
        self.assertFalse(rolodexer.is_color('James'))
        self.assertFalse(rolodexer.is_color('Murphy'))
        self.assertFalse(rolodexer.is_color('87360'))
        self.assertFalse(rolodexer.is_color('373 781 7380'))
        self.assertFalse(rolodexer.is_color('(623)-668-9293'))
        self.assertFalse(rolodexer.is_color('123123121'))
        self.assertFalse(rolodexer.is_color('83880'))
        self.assertFalse(rolodexer.is_color('018 154 6474'))
    
    def test_identify_phone(self):
        self.assertTrue(rolodexer.is_phone('373 781 7380'))
        self.assertTrue(rolodexer.is_phone('(623)-668-9293'))
        self.assertTrue(rolodexer.is_phone('018 154 6474'))
        
        # this should (problematically) read as a phone number
        # self.assertTrue(rolodexer.is_phone('123123121'))
        
        self.assertFalse(rolodexer.is_phone('Booker T.'))
        self.assertFalse(rolodexer.is_phone('Washington'))
        self.assertFalse(rolodexer.is_phone('Chandler'))
        self.assertFalse(rolodexer.is_phone('Kerri'))
        self.assertFalse(rolodexer.is_phone('James'))
        self.assertFalse(rolodexer.is_phone('Murphy'))
        self.assertFalse(rolodexer.is_phone('87360'))
        self.assertFalse(rolodexer.is_phone('83880'))
        self.assertFalse(rolodexer.is_phone('yellow'))
        self.assertFalse(rolodexer.is_phone('pink'))
        self.assertFalse(rolodexer.is_phone('aqua marine'))
    
    def test_identify_zip(self):
        self.assertTrue(rolodexer.is_zip('83880'))
        self.assertTrue(rolodexer.is_zip('87360'))
        # self.assertTrue(rolodexer.is_zip('02459-1234'))
        
        self.assertFalse(rolodexer.is_zip('Booker T.'))
        self.assertFalse(rolodexer.is_zip('Washington'))
        self.assertFalse(rolodexer.is_zip('Chandler'))
        self.assertFalse(rolodexer.is_zip('Kerri'))
        self.assertFalse(rolodexer.is_zip('James'))
        self.assertFalse(rolodexer.is_zip('Murphy'))
        self.assertFalse(rolodexer.is_zip('pink'))
        self.assertFalse(rolodexer.is_zip('aqua marine'))
        self.assertFalse(rolodexer.is_zip('373 781 7380'))
        self.assertFalse(rolodexer.is_zip('(623)-668-9293'))
        self.assertFalse(rolodexer.is_zip('123123121'))
        self.assertFalse(rolodexer.is_zip('yellow'))
        self.assertFalse(rolodexer.is_zip('018 154 6474'))
    
    def test_classify(self):
        terms = [
            u'yellow', u'373 781 7380', u'87360',
            u'Washington', u'Booker T.']
        
        out = rolodexer.classify(terms)
        keys = out.keys()
        
        self.assertTrue(u'phonenumber' in keys)
        self.assertTrue(u'firstname' in keys)
        self.assertTrue(u'lastname' in keys)
        self.assertTrue(u'color' in keys)
        self.assertTrue(u'zipcode' in keys)
        
        self.assertEqual(out[u'color'],          terms[0])
        self.assertEqual(out[u'phonenumber'],    rolodexer.phone_format(terms[1]))
        self.assertEqual(out[u'zipcode'],        terms[2])
        self.assertEqual(out[u'lastname'],       terms[3])
        self.assertEqual(out[u'firstname'],      terms[4])
    
    def _test_bad_line_raises(self):
        """ assertRaises() is holding some sort of grudge
            against my entire bloodline, for some reason
        """
        # from rolodexer import RDZipCodeError, RolodexerError
        lines = sample_input.splitlines()
        for idx, line in enumerate(lines):
            terms = rolodexer.tokenize(line)
            # with self.assertRaises(RDZipCodeError):
            self.assertRaises(
                Exception, 
                rolodexer.classify, terms)
    
    def test_tokenize_classify(self):
        # from pprint import pprint
        entries = []
        errors  = []
        lines = sample_input.splitlines()
        for idx, line in enumerate(lines):
            terms = rolodexer.tokenize(line)
            try:
                cterms = rolodexer.classify(terms)
            except rolodexer.RolodexerError:
                errors.append(idx)
            else:
                keys = cterms.keys()
                
                self.assertTrue(u'phonenumber' in keys)
                self.assertTrue(u'firstname' in keys)
                self.assertTrue(u'lastname' in keys)
                self.assertTrue(u'color' in keys)
                self.assertTrue(u'zipcode' in keys)
                
                entries.append(cterms)
        
        output_dict = { u"entries": entries, u"errors": errors }
        # pprint(output_dict)
        
        sample_output_dict = json.loads(sample_output)
        self.assertItemsEqual(
            output_dict, sample_output_dict)
    
    def test_json_format(self):
        pass
    
    def test_file_read(self):
        

if __name__ == '__main__':
    unittest.main()

if __name__ == '__yodogg__':
    line = sample_input.splitlines()[0]
    
    while True:
        partition   = line.rpartition(',')
        first       = partition[:-2][0].strip()
        last        = partition[-1:][0].strip()
        if not first and not last:
            break
        line = first
        print last



#!/usr/bin/env python
"""
Usage:
  rolodexer INFILE [-o OUTFILE] [-V | --verbose]
  rolodexer -h | --help | -v | --version
    
Options:
  -o OUTFILE --output=OUTFILE       specify output file [default: stdout]
  -V --verbose                      print verbose output
  -h --help                         show this text
  -v --version                      print version

"""

from __future__ import print_function
# from os.path import join, dirname
from os.path import exists, isdir, dirname
from docopt import docopt
from rolodexer.histogram import Histogram
import parser as rolodexer
import json
import sys

JSON_ARGS = dict(indent=2, sort_keys=True)

def print_colors(histo):
    print("Colors: %s" % len(histo), file=sys.stderr)
    for color, count in histo.iteritems():
        print("\t%15s\t\t%s" % (color, int(count)), file=sys.stderr)
    print('', file=sys.stderr)

def cli(argv=None):
    if not argv:
        argv = sys.argv
    
    arguments = docopt(__doc__, argv=argv[1:],
                                help=True,
                                version='0.1.3')
    
    # print(argv)
    # print(arguments)
    # sys.exit()
    
    entries = []
    errors  = []
    colors  = Histogram()
    
    ipth = arguments.get('INFILE')
    opth = arguments.get('--output')
    verbose = bool(arguments.get('--verbose'))
    
    with open(ipth, 'rb') as fh:
        idx = 0
        while True:
            linen = fh.readline()
            if not linen:
                break
            line = linen.strip()
            tokens = rolodexer.tokenize(line)
            try:
                terms = rolodexer.classify(tokens)
            except rolodexer.RolodexerError:
                errors.append(idx)
            else:
                entries.append(terms)
                if 'color' in terms:
                    colors.inc(terms.get('color'))
            idx += 1
        
        output_dict = { u"entries": entries, u"errors": errors }
        
        if verbose:
            print("Entries parsed: %s" % len(entries), file=sys.stderr)
            print("Errors encountered: %s" % len(errors), file=sys.stderr)
            print_colors(colors)
        
        if opth == 'stdout':
            output_json = json.dumps(output_dict, **JSON_ARGS)
            print(output_json, file=sys.stdout)
        elif not exists(opth) and isdir(dirname(opth)):
            if verbose:
                print("rolodexer: saving output to %s" % opth, file=sys.stderr)
            with open(opth, 'wb') as fp:
                json.dump(output_dict, fp, **JSON_ARGS)

if __name__ == '__main__':
    cli(sys.argv)



from __future__ import print_function

import re
import phonenumbers

from copy import copy
# from pprint import pprint
from rolodexer.histogram import Histogram

class RolodexerError(Exception): pass
class RDAmbiguousTerms(RolodexerError): pass
class RDAmbiguousNames(RolodexerError): pass
class RDAmbiguousNumber(RolodexerError): pass
class RDPhoneNumberError(RolodexerError): pass
class RDZipCodeError(RolodexerError): pass

# simple line assumptions
SEP     = ','
SEP_WS  = ', '
D = re.compile(r'\D+') # kill non-digits
digify = lambda term: D.subn('', term)[0]

# simple term assumptions
is_zip          = lambda term: len(digify(term)) == 5
is_phone        = lambda term: len(digify(term)) == 10
is_color        = lambda term: term.islower()

# format according to standards
def phone_format(term):
    return phonenumbers.format_number(
        phonenumbers.parse(term, 'US'),
        phonenumbers.PhoneNumberFormat)

def tokenize(line_input):
    """ basically: r"(name), (zip|phone|color),
                             (zip|phone|color),
                             (zip|phone|color)"
    """
    terms  = []
    line    = unicode(line_input)
    
    while True:
        partition   = line.rpartition(',')
        first       = partition[:-2][0].strip()
        last        = partition[-1:][0].strip()
        if not first and not last:
            break
        line = first
        terms.append(last)
    return terms

def reconstruct(terms):
    return SEP_WS.join(reversed(terms))

def classify(orig_terms):
    out = dict()
    terms = copy(orig_terms)
    
    # first, sanity-check the digified terms --
    # if more than one can pass for a phone number, a color,
    # or a zip code (that is to say, the input is ambiguous),
    # we bail:
    for term in terms:
        # check each term against all test funcs --
        # if more than one bucket is nonzero, it's a problem
        h = Histogram()
        if is_zip(term):
            h.inc('zip')
        if is_phone(term):
            h.inc('phone')
        if is_color(term):
            h.inc('color')
        if len(h) > 1:
            # ERROR: couldn't distinguish one thing
            # from another... BAIL
            raise RDAmbiguousTerms("Term '%s' parsed ambiguously\n"
                                   "Passed multiple tests: %s" % (
                                       term, SEP_WS.join(h.iterkeys())
                                   ))
    
    # next, recurse and grab the phone number and color
    # ... they are the easiest to find:
    for idx, term in enumerate(copy(terms)):
        # tref = terms[idx] # I do miss C++ sometimes
        if is_phone(term):
            out.update({ u'phonenumber': u"%s" % phone_format(term) })
            terms.remove(term)
            continue
        elif is_color(term):
            out.update({ u'color': u"%s" % term })
            terms.remove(term)
            continue
        elif is_zip(term):
            out.update({ u'zipcode': u"%s" % term })
            terms.remove(term)
            continue
    
    if not out.has_key(u'phonenumber'):
        # ERROR: NO PHONE / BAD PHONE!
        raise RDPhoneNumberError("No valid phone number in %d-term list\n"
                                 "Reconstructed original line:\n"
                                 "\t%s" % (len(terms), reconstruct(orig_terms)))
    
    if not out.has_key(u'zipcode'):
        # ERROR: NO ZIPCODE / BAD ZIPCODE!
        raise RDZipCodeError("No valid zip code in %d-term list\n"
                             "Reconstructed original line:\n"
                             "\t%s" % (len(terms), reconstruct(orig_terms)))
    
    if not out.has_key(u'color'):
        # LESS DISCONCERTING ERROR: NO COLOR / BAD COLOR!
        pass
    
    # what is left "should" be the pieces of the name,
    # e.g. ['Washington', 'Booker T.'], ['James Murphy'], &c
    if len(terms) > 2:
        # ERROR: wtf is going on
        pass
    elif len(terms) == 2:
        out.update({ 
            u'firstname':    u"%s" % terms[-1],
            u'lastname':     u"%s" % terms[0]
        })
    elif len(terms) == 1:
        names = terms[0].split()
        if len(names) > 1:
            out.update({
                u'firstname':    u"%s" % names[0],
                u'lastname':     u"%s" % names[-1]
            })
        else:
            # ERROR: only one name -- `raise MadonnaError()` ?
            # ... use it as the *last* name for now, maybe
            # ... naw, f that: ERROR.
            raise RDAmbiguousNames("Only one name present: '%s'\n"
                                   "Reconstructed original line:\n"
                                   "\t%s" % (names.pop(), reconstruct(orig_terms)))
        
    else:
        # WHY ARE WE HERE. No names... really??
        raise RDAmbiguousNames("No names present!"
                               "Reconstructed original line:\n"
                               "\t%s" % reconstruct(orig_terms))
        
    # pprint(out, indent=4)
    return out




import re
import phonenumbers

from copy import copy
from histogram import Histogram

SEP     = ','
SEP_WS  = ', '
D = re.compile(r'\D+')
FIELD_DMV = []

class RolodexerError(Exception): pass
class RDAmbiguousTerms(RolodexerError): pass
class RDAmbiguousNames(RolodexerError): pass
class RDAmbiguousNumber(RolodexerError): pass
class RDPhoneNumberError(RolodexerError): pass
class RDZipCodeError(RolodexerError): pass

def reconstruct(tokens):
    return SEP_WS.join(reversed(tokens))

class fieldtype(type):
    def __new__(cls, name, bases, attrs):
        # super_new bit cribbed from Django:
        super_new = super(fieldtype, cls).__new__
        # parents = [b for b in bases if isinstance(b, fieldtype)]
        newcls = super_new(cls, name, bases, attrs)
        if newcls.json_name != 'field':
            FIELD_DMV.append(newcls)
        return newcls
    
    def digify(cls, token):
        return D.subn('', token)[0]
    
    def check(cls, token):
        raise NotImplementedError()


class FieldBase(object):
    json_name = u'field'
    
    @property
    def name(self):
        return self.json_name
    
    def format(self, token):
        return u"%s" % token
    
    def value_for_token(self, token):
        return { self.name: self.format(token) }


class Field(FieldBase):
    __metaclass__ = fieldtype
    
    def unfound(self, token_list):
        pass # Specifically, do not raise here


class ZipcodeField(Field):
    json_name = 'zipcode'
    
    class __metaclass__(fieldtype):
        def check(cls, token):
            return len(cls.digify(token)) == 5
    
    def unfound(self, token_list):
        raise RDZipCodeError("No valid zip code in %d-term list\n"
                             "Reconstructed original line:\n"
                             "\t%s" % (len(token_list), reconstruct(token_list)))


class ColorField(Field):
    
    json_name = 'color'
    
    class __metaclass__(fieldtype):
        def check(cls, token):
            return token.islower()


class PhoneNumberField(Field):
    json_name = 'phonenumber'
    
    class __metaclass__(fieldtype):
        def check(cls, token):
            return len(cls.digify(token)) == 10
    
    def format(self, token):
        return u"%s" % phonenumbers.format_number(
            phonenumbers.parse(token, 'US'),
            phonenumbers.PhoneNumberFormat)
    
    def unfound(self, token_list):
        raise RDPhoneNumberError("No valid phone number in %d-term list\n"
                                 "Reconstructed original line:\n"
                                 "\t%s" % (len(token_list), reconstruct(token_list)))

class NameField(FieldBase):
    def unfound(self, token_list):
        raise RDAmbiguousNames("Only one name present!\n"
                               "Reconstructed original line:\n"
                               "\t%s" % reconstruct(token_list))

class FirstNameField(NameField):
    """ Special case for first-name fields """
    json_name = u'firstname'

class LastNameField(NameField):
    """ Special case for last-name fields """
    json_name = u'lastname'


def tokenize(line_input):
    tokens  = []
    line    = unicode(line_input)
    
    while True:
        partition   = line.rpartition(',')
        first       = partition[:-2][0].strip()
        last        = partition[-1:][0].strip()
        if not first and not last:
            break
        line = first
        tokens.append(last)
    return tokens


def classify(token_list):
    out = dict()
    tokens = copy(token_list)
    
    # from pprint import pprint
    # pprint(FIELD_DMV)
    
    # first, sanity-check the digified tokens --
    # if more than one can pass for a phone number, a color,
    # or a zip code (that is to say, the input is ambiguous),
    # we bail:
    for token in tokens:
        # check each term against all test funcs --
        # if more than one bucket is nonzero, it's a problem
        h = Histogram()
        for FieldType in FIELD_DMV:
            if FieldType.check(token):
                h.inc(FieldType.json_name)
        if len(h) > 1:
            # ERROR: couldn't distinguish one thing
            # from another... BAIL
            raise RDAmbiguousTerms("Token '%s' parsed ambiguously\n"
                                   "Passed multiple field checks: %s" % (
                                       token, SEP_WS.join(h.iterkeys())
                                   ))
    
    # update `out` with the classified tokens
    for idx, token in enumerate(copy(tokens)):
        for FieldType in FIELD_DMV:
            if FieldType.check(token):
                field = FieldType()
                out.update(field.value_for_token(token))
                tokens.remove(token)
            continue
    
    # raise appropriate errors when we don't find what we need
    for FieldType in FIELD_DMV:
        field = FieldType()
        if not out.has_key(field.name):
            field.unfound(token_list) # this may raise
    
    # what is left "should" be the pieces of the name,
    # e.g. ['Washington', 'Booker T.'], ['James Murphy'], &c
    first_field = FirstNameField()
    last_field = LastNameField()
    if len(tokens) > 2:
        # ERROR: wtf is going on
        pass
    elif len(tokens) == 2:
        out.update(first_field.value_for_token(tokens[-1]))
        out.update(last_field.value_for_token(tokens[0]))
    elif len(tokens) == 1:
        names = tokens[0].split()
        if len(names) > 1:
            out.update(first_field.value_for_token(names[0]))
            out.update(last_field.value_for_token(names[-1]))
        else:
            NameField().unfound(token_list)
    else:
        # WHY ARE WE HERE. No names... really??
        raise RDAmbiguousNames("No names present!"
                               "Reconstructed original line:\n"
                               "\t%s" % reconstruct(token_list))
    
    # pprint(out, indent=4)
    return out





from pkgutil import extend_path
__path__ = extend_path(__path__, __name__)

from parser import (
    tokenize, classify,
    
    RolodexerError,
    RDAmbiguousTerms,
    RDAmbiguousNames,
    RDAmbiguousNumber,
    RDPhoneNumberError,
    RDZipCodeError,
    
    ZipcodeField,
    ColorField,
    PhoneNumberField
)


#!/usr/bin/env python

fish2000 = 'fish2000'
name = 'rolodexer'
long_name = 'rolodexer'
version = '0.1.3'
packages = [name]
description = "Parse rolodex data and output JSON"

keywords = [
    'JSON','CSV','rolodex','contact','database','parse'
]

long_description = """
    
    Rolodexer - parse comma-separated "rolodex" data files and output JSON.
    
"""


classifiers = [
    'Development Status :: 5 - Production/Stable',
    'Environment :: Other Environment',
    'Environment :: Plugins',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Topic :: Database',
    'Topic :: Utilities',
]

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

import sys

if 'sdist' in sys.argv and 'upload' in sys.argv:
    """ CLEAN THIS MESS UP RIGHT NOW YOUNG MAN """
    import commands
    import os
    finder = "/usr/bin/find %s \( -name \*.pyc -or -name .DS_Store \) -delete"
    theplace = os.getcwd()
    if theplace not in (".", "/"):
        print("+ Deleting crapola from %s..." % theplace)
        print("$ %s" % finder % theplace)
        commands.getstatusoutput(finder % theplace)
        print("")

setup(

    name=long_name, version=version, description=description,
    long_description=long_description,
    download_url=('https://github.com/%s/%s/archive/master.zip' % (fish2000, name)),

    author=u"Alexander Bohn",
    author_email='%s@gmail.com' % fish2000,
    url='http://github.com/%s/%s' % (fish2000, name),
    license='GPLv2',
    keywords=', '.join(keywords),
    platforms=['any'],
    
    include_package_data=True,
    
    packages=[]+packages,
    
    package_dir={
        'rolodexer': 'rolodexer',
    },
    
    entry_points={
        'console_scripts': [
            'rolodexer = rolodexer.cli:cli'
        ],
    },
    
    package_data={},
    install_requires=['phonenumbers', 'docopt'],

    classifiers=classifiers+[
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: MacOS',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: OS Independent',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: Implementation :: PyPy'],
)




#!/usr/bin/env python
from __future__ import print_function

from collections import defaultdict, namedtuple
from operator import add
from math import floor

class AutoInc(object):
    # ''' Get a uniquely auto-incremented integer value '''
    __slots__ = ('v',)
    
    def __init__(self, value=0):
        self.v = int(value)
    
    def __call__(self):
        self.v += 1
        return self.v
    
    def add(self, value):
        self.v += int(value)
        return self.v
    
    def __int__(self):
        return self.v
    
    def __float__(self):
        return float(self.v)
    
    def __long__(self):
        return long(self.v)
    
    def __repr__(self):
        return str(self.v)
    
    def __str__(self):
        return str(self.v)

class Histogram(defaultdict):
    
    @staticmethod
    def lexcmp(k0, k1):
        """ Hacky comparator to privilege string length """
        return cmp(len(k1), len(k0)) or cmp(k0, k1)
    
    @staticmethod
    def autoinc():
        return AutoInc()
    
    def __init__(self, *args, **kwargs):
        for arg in args:
            argclsname = arg.__class__.__name__
            if argclsname.startswith('FrozenHistogram'):
                # reconstitute FrozenHistogram
                kwargs.update(arg._asdict())
            elif (argclsname.startswith('Histogram') or
                  argclsname.startswith('defaultdict') or
                  argclsname.startswith('dict')):
                # copy-construct
                kwargs.update(dict(arg))
            else:
                # normal arg: column name
                kwargs.update({ str(arg): AutoInc() })
        super(Histogram, self).__init__(AutoInc, *tuple(), **kwargs)
    
    def prettyprint(self, **kwargs):
        from pprint import pformat
        return "%s(%s,\n%s)" % (
            self.__class__.__name__,
            AutoInc.__name__,
            pformat(dict(self), **kwargs))
    
    def __repr__(self):
        return self.prettyprint()
    
    def __str__(self):
        longest_key = sorted(self.iterkeys(),
                             cmp=Histogram.lexcmp,
                             reverse=False)[0]
        return self.prettyprint(depth=10,
                                width=len(longest_key))
    
    def inc(self, idx):
        return self[str(idx)]()
    
    def add(self, idx, value):
        return self[str(idx)].add(value)
    
    def val(self, idx):
        return int(self[idx])
    
    def keyset_hash(self):
        return hex(abs(hash(self.iterkeys())))
    
    def frozentype(self, rename=True):
        return namedtuple('FrozenHistogram_%s' % self.keyset_hash(),
            tuple(self.iterkeys()), rename=rename)
    
    def freeze(self):
        return self.frozentype()(**self)
    
    def normalize(self):
        ceil = float(sum(int(v) for v in self.itervalues()))
        return self.frozentype()(**dict(
            (ituple[0], float(ituple[1]) / ceil) for ituple in self.iteritems()
        ))
    
    def max(self):
        """ Maximum value """
        return max(*(int(v) for v in self.values()))
    
    def min(self):
        """ Minimum value """
        return min(*(int(v) for v in self.values()))
    
    def sum(self):
        return reduce(add, (int(v) for v in self.itervalues()))
    
    def average(self):
        """ Histogram mean/average (float) """
        return float(self.sum()) / float(len(self))
    
    def avg(self):
        """ Histogram mean/average (integer) """
        return int(floor(self.average()))
    
    def range(self):
        """ Histogram numerical range (integer) """
        return self.max() - self.min()
    
    def valuelist(self):
        return sorted(int(v) for v in self.itervalues())
    
    def median(self):
        """ Median histogram value (integer, or int/2) """
        vals = self.valuelist()
        idx = len(vals) / 2
        if len(vals) % 2:
            return vals[idx+1]
        return float(vals[idx] + vals[idx+1]) / 2.0
    
    def keys_for_value(self, value):
        """ List of keys with the given integer value """
        return map(lambda pair: pair[0],
            filter(lambda pair: int(pair[1]) == int(value),
            self.iteritems()))
    
    def valueset(self):
        return set(self.valuelist())

if __name__ == '__main__':
    # Hh = Histogram('color', 'phone', 'zip', 'firstname', 'lastname')
    Hh = Histogram('color', 'phone', 'zip', firstname=10, lastname=10)
    Hp = Histogram(firstname=0, lastname=0)
    
    Hh['color']()
    Hh['color']()
    Hh['color']()
    Hh['color']()
    Hh['color']()
    Hh['color']()
    
    Hh['phone']()
    Hh['phone']()
    Hh['phone']()
    Hh['phone']()
    Hh['phone']()
    Hh['phone']()
    Hh['phone']()
    Hh['phone']()
    Hh['phone']()
    Hh['phone']()
    Hh['phone']()
    Hh['phone']()
    Hh['phone']()
    Hh['phone']()
    Hh['phone']()
    Hh['phone']()
    Hh['phone']()
    Hh['phone']()
    Hh['phone']()
    
    Hh['zip']()
    Hh['zip']()
    Hh['zip']()
    Hh['zip']()
    
    print(Hh)
    print(Hh.freeze()._asdict())
    print(Hh.normalize())
    print('')
    
    # print("MAX Hh: %s" % max(*(int(v) for v in Hh.values())))
    print("MIN Hh: %s" % Hh.min())
    print("MAX Hh: %s" % Hh.max())
    print("SUM Hh: %s" % Hh.sum())
    print("AVG Hh: %s" % Hh.avg())
    print("RANGE Hh: %s" % Hh.range())
    print("MEDIAN Hh: %s" % Hh.median())
    print("KEYS IN Hh FOR 19: %s" % Hh.keys_for_value(19))
    print("KEYS IN Hh FOR 777: %s" % Hh.keys_for_value(777))
    print("KEYS IN Hh FOR 0: %s" % Hh.keys_for_value(0))
    print('')
    
    hh = Histogram(Hh, 'iheard', 'youlike', yo=10, dogg=100)
    
    print(hh)
    print(hh.freeze()._asdict())
    print(hh.normalize())
    print('')
    
    print("MIN hh: %s" % hh.min())
    print("MAX hh: %s" % hh.max())
    print("SUM hh: %s" % hh.sum())
    print("AVG hh: %s" % hh.avg())
    print("RANGE hh: %s" % hh.range())
    print("MEDIAN hh: %s" % hh.median())
    print("KEYS IN hh FOR 19: %s" % hh.keys_for_value(19))
    print("KEYS IN hh FOR 777: %s" % hh.keys_for_value(777))
    print("KEYS IN hh FOR 0: %s" % hh.keys_for_value(0))
    print('')
    
    Hp.inc('color')
    Hp.inc('color')
    Hp.inc('color')
    Hp.inc('color')
    Hp.inc('color')
    Hp.inc('color')
    
    Hp.inc('zip')
    Hp.inc('zip')
    
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    Hp.inc('phone')
    
    print(Hp)
    
    print(Hp.freeze()._asdict())
    print(Hp.normalize())
    
    print("MAX Hp: %s" % Hp.max())
